//
// Universal very-weak automaton tool set - using BDDs
//

#ifndef __UVW_H__
#define __UVW_H__

#include <vector>
#include <BF.h>

//=================================
// UVW State data structure
//=================================
struct UVWState {
    bool isFinal;
    bool isInitial;
    bool isActive; // Set to false after removing a state
    std::vector<std::pair<uint,BF> > transitions;
public:
    UVWState() : isFinal(false), isInitial(false), isActive(true) {}
    BF getSelfLoop(uint stateNr) {
        if (transitions.size()==0) throw "getSelfLoop is *only* supported on states that actually have a self-loop";
        BF loopCondition = transitions[0].second.manager()->constantFalse();
        for (std::vector<std::pair<uint,BF> >::iterator it = transitions.begin();it!=transitions.end();it++) {
            if (it->first==stateNr) loopCondition |= it->second;
        }
        return loopCondition;
    }
};

struct UVW {
    // Data
    std::vector<UVWState> states;
    UVW() {}
    BF getSelfLoop(uint stateNr) {
        return states[stateNr].getSelfLoop(stateNr);
    }
};

#endif
