// Advanced VariableInfo Container
//
// Provides variable groups and the like
#ifndef __ADVANCED_VARIABLE_INFO_CONTAINER_HPP__
#define __ADVANCED_VARIABLE_INFO_CONTAINER_HPP__

#include "bddDump.h"
#include <map>

class AdvancedVariableInfoContainer : public VariableInfoContainer {

protected:

    // General BF stuff
    BFManager mgr;
    std::vector<BF> vars;
    std::vector<std::string> varNames;

    class VariableGroup {
    private:
        std::vector<uint> variableNumbers;
        std::vector<BF> variables;
        BFVarCube cube;
        BFVarVector varVector;
        AdvancedVariableInfoContainer *parent;
    public:
        VariableGroup(AdvancedVariableInfoContainer *_parent) : parent(_parent) {
            cube = parent->mgr.computeCube(variables);
            varVector = parent->mgr.computeVarVector(variables);
        }
        void add(uint nr) {
            variableNumbers.push_back(nr);
            variables.push_back(parent->vars[nr]);
            cube = parent->mgr.computeCube(variables);
            varVector = parent->mgr.computeVarVector(variables);
        }
        BFVarCube getCube() {
            return cube;
        }
        BFVarVector getVector() {
            return varVector;
        }
        std::vector<BF> getVarBFs() {
            return variables;
        }
        uint getVarNumber(uint index) {
            return variableNumbers[index];
        }
        BF getVarBF(uint nr) {
            return variables[nr];
        }
        uint size() const {
            return variables.size();
        }

        // Access iterators
        std::vector<uint>::const_iterator begin() const {
            return variableNumbers.begin();
        }
        std::vector<uint>::const_iterator end() const {
            return variableNumbers.end();
        }

        operator const std::vector<uint> () { return variableNumbers; };

        uint operator [] (uint index) const {
            return variableNumbers[index];
        }
    };

    std::map<std::string,VariableGroup*> variableGroups;

public:
    AdvancedVariableInfoContainer() : mgr(4096,1.05) {}


};









#endif
