//
// A variable allocation optimier.
// Solves the following optimization problem:
//
// Given a set X and a Multiset Y of subsets of X
// Find a subset S of X of minimal cost
// The cost is defined as the sum of
// - b*|S|
// - c for every element of Y for which not all of its elements are in S
//
// The solution is based on network flow/minimum cut analysis

#ifndef __VARIABLE__ALLOCATION_OPTIMIZER_HPP__
#define __VARIABLE__ALLOCATION_OPTIMIZER_HPP__

#include <vector>
#include <set>
#include <stdlib.h>
#include <iostream>

class VariableAllocationOptimizer {
private:
    uint nofVariables;
    int penaltyUseVar; // The variable "b" in the explanation above
    int penaltyNotUseCombination; // The variable "c" in the explanaition above
    std::vector<std::set<uint> > combinations;
    std::vector<bool> solutionVariables;
    std::vector<bool> solutionCombinations;


public:
    VariableAllocationOptimizer(uint _nofVariables, int _penaltyUseVar, int _penaltyNotUseCombination) : nofVariables(_nofVariables), penaltyUseVar(_penaltyUseVar), penaltyNotUseCombination(_penaltyNotUseCombination) {
    }

    void addCombination(std::vector<uint> &combination) {
        combinations.push_back(std::set<uint>(combination.begin(),combination.end()));
    }

    void addCombination(std::set<uint> &combination) {
        combinations.push_back(combination);
    }

    std::vector<bool> const & getSolutionVariables() const {
        return solutionVariables;
    }

    std::vector<bool> const & getSolutionCombinations() const {
        return solutionCombinations;
    }

    std::set<uint> getCombination(uint nr) {
        return combinations[nr];
    }

    void solve();

};


#endif
