/*
 * BFStateToVariableValuationEncoder.h
 *
 *  Created on: 06.08.2010
 *      Author: ehlers
 */

#ifndef BFSTATETOVARIABLEVALUATIONENCODER_H_
#define BFSTATETOVARIABLEVALUATIONENCODER_H_

#include <cstdlib>

/**
 * A class that takes some transition structure and generates efficient assigments of
 * BF variable valuations for every state in order to facilitate building a symbolic
 * transition relation.
 *
 * Example: Given a transition relation {(0,1),(1,2),(2,0)} for a three-state automaton,
 * a variable valuation could be:
 *
 * 0 -> 00
 * 1 -> 01
 * 2 -> 10
 *
 * The latter numbers are in binary representation.
 *
 */
class BFStateToVariableValuationEncoder {
public:
	virtual ~BFStateToVariableValuationEncoder() {};

	virtual void run() = 0;
	virtual void reset(uint numberOfStates) = 0;
	virtual void addTransition(uint from, uint to) = 0;
	virtual uint getAssignment(uint stateNr) const = 0;
	virtual int getNumberOfBits() const = 0;

};

#endif /* BFSTATETOVARIABLEVALUATIONENCODER_H_ */
