#! /usr/bin/env python
# -*- coding: ISO-8859-15 -*-
# 
from __future__ import with_statement
import os, glob, sys, tempfile

# Compute path to examples
basepath =  os.path.realpath(__file__)
basepath = basepath[0:basepath.rfind("/Tools/")]
print >>sys.stderr, "Base path: "+basepath

# Accept files from the command line or use set of benchmarks instead
if len(sys.argv)>1:
    inputFile = sys.argv[1]
else:
    print >>sys.stderr, "Error: You must prove an input XML specification."
    sys.exit(1)

# Output model wanted?
if len(sys.argv)>2:
    outputFile = sys.argv[2]
else:
    outputFile = None

# Allocate temporary directory
tempdir = tempfile.mkdtemp(".bassist")

# A function for cleaning up the temporary directory after we are done
def cleanUpTempDir(tempdir):
    if not os.environ.has_key("NOCLEANUP"):
        for myfile in ["intermediate","intermediate2","synSpec","nuSMVVerificationObligations.txt","model.txt"]:
            try:        
                os.remove(tempdir+os.sep+myfile)
            except OSError:
                # Ok, file probably wasn't there. Then it's all right...
                pass
        os.rmdir(tempdir)

# Parse all LTL formulas
parseCmd = basepath+"/Tools/ParseLTL/parseFormulasInXMLSpecFile.py "+inputFile+" > "+tempdir+os.sep+"intermediate"
print >>sys.stderr, " -- "+parseCmd
sys.stdout.flush()
return_code = os.system(parseCmd)

if return_code>0:
    print "Error: Could not parse the input instance."
    cleanUpTempDir(tempdir)
    sys.exit(100)

# For reference, compute NUSMV verification condition. This also checks if the all APs used are declared.
parseCmd = basepath+"/Tools/ParseLTL/generateNuSMVVerificationObligations.py "+tempdir+os.sep+"intermediate > "+tempdir+os.sep+"nuSMVVerificationObligations.txt"
print >>sys.stderr, " -- "+parseCmd
sys.stdout.flush()
return_code = os.system(parseCmd)

if return_code>0:
    print "Error: Could not parse the input instance."
    cleanUpTempDir(tempdir)
    sys.exit(100)


# Convert all formulas to Vermicelli
toVermicelliCmd = basepath+"/Tools/ParseLTL/convertAllSpecsToVermicelliSets.py "+tempdir+os.sep+"intermediate > "+tempdir+os.sep+"intermediate2"
print >>sys.stderr, " -- "+toVermicelliCmd
sys.stdout.flush()
return_code = os.system(toVermicelliCmd)

if return_code>0:
    print "Error: Could not perform first automata conversion step. Probably the input specification you provided is not in the supported fragment."
    cleanUpTempDir(tempdir)
    sys.exit(100)


# Convert all formulas to UVW
toUVWCmd = basepath+"/Tools/VermicelliSetsToUVW/converter "+tempdir+os.sep+"intermediate2 > "+tempdir+os.sep+"synSpec"
print >>sys.stderr, " -- "+toUVWCmd
sys.stdout.flush()
return_code = os.system(toUVWCmd)

if return_code>0:
    print "Error: Could not build automta. Typically, this is an internal error."
    cleanUpTempDir(tempdir)
    sys.exit(100)


# Perform synthesis operation
synthesisCmd = basepath+"/Tools/BDDBackend/backend "+tempdir+os.sep+"synSpec > "+tempdir+os.sep+"model.txt"
print >>sys.stderr, " -- "+synthesisCmd
sys.stdout.flush()
return_code = os.system(synthesisCmd)

if return_code>0:
    # cleanUpTempDir(tempdir)
    sys.exit(return_code/256)


# Success - Complete model if wanted!
if outputFile!=None:
    with open(outputFile,"w") as outHandle:
        with open(tempdir+os.sep+"model.txt","r") as inputFile:
            for line in inputFile.readlines():
                outHandle.write(line)
        with open(tempdir+os.sep+"nuSMVVerificationObligations.txt","r") as inputFile:
            for line in inputFile.readlines():
                outHandle.write(line)

    print "Generated model: "+outputFile
        
cleanUpTempDir(tempdir)
sys.exit(0)
